/**
 * @file
 * n8n Chat JavaScript functionality.
 */

(function ($, Drupal, drupalSettings) {
  'use strict';

  /**
   * Generate a unique session ID for the chat session.
   */
  function generateSessionId() {
    return 'chat_' + Date.now() + '_' + Math.random().toString(36).substr(2, 9);
  }

  /**
   * Get or create session ID from localStorage.
   */
  function getSessionId() {
    var sessionId = localStorage.getItem('n8n_chat_session_id');
    if (!sessionId) {
      sessionId = generateSessionId();
      localStorage.setItem('n8n_chat_session_id', sessionId);
      localStorage.setItem('n8n_chat_session_timestamp', Date.now().toString());
    }
    return sessionId;
  }

  /**
   * Force refresh session ID (creates new session on page load).
   */
  function refreshSessionId() {
    var newSessionId = generateSessionId();
    localStorage.setItem('n8n_chat_session_id', newSessionId);
    localStorage.setItem('n8n_chat_session_timestamp', Date.now().toString());
    return newSessionId;
  }

  /**
   * Reset the session ID (useful for starting a new conversation).
   */
  function resetSessionId() {
    localStorage.removeItem('n8n_chat_session_id');
    localStorage.setItem('n8n_chat_session_timestamp', Date.now().toString());
    var newSessionId = generateSessionId();
    localStorage.setItem('n8n_chat_session_id', newSessionId);
    return newSessionId;
  }

  /**
   * Check if session is expired (older than 24 hours) and reset if needed.
   */
  function checkSessionExpiry() {
    var sessionTimestamp = localStorage.getItem('n8n_chat_session_timestamp');
    var now = Date.now();
    var twentyFourHours = 24 * 60 * 60 * 1000; // 24 hours in milliseconds

    if (!sessionTimestamp || (now - parseInt(sessionTimestamp)) > twentyFourHours) {
      resetSessionId();
      localStorage.setItem('n8n_chat_session_timestamp', now.toString());
    }
  }

  /**
   * n8n Chat behavior.
   */
  Drupal.behaviors.n8nChat = {
    attach: function (context, settings) {
      // Only initialize once
      if (context !== document) {
        return;
      }

      var widgetSettings = drupalSettings.n8nChat || {};

      // Handle session behavior based on configuration
      var sessionBehavior = widgetSettings.sessionBehavior || 'refresh';

      if (sessionBehavior === 'refresh') {
        // Always start new session
        refreshSessionId();
      } else if (sessionBehavior === 'persist') {
        // Continue previous session (just ensure we have a session ID)
        getSessionId();
      } else if (sessionBehavior === 'load_history') {
        // Continue session and load history
        getSessionId();
      }

      // Create chat widget if webhook URL is configured
      if (widgetSettings.webhookUrl) {
        createChatWidget(widgetSettings);
      }
    }
  };

  /**
   * Creates the n8n chat widget.
   */
  function createChatWidget(settings) {
    // Check if widget already exists
    if ($('#n8n-chat').length > 0) {
      return;
    }

    var widgetHtml = `
      <div id="n8n-chat" class="n8n-chat ${settings.widgetPosition} ${settings.widgetTheme}">
        <div class="chat-toggle" id="chat-toggle">
          <span class="chat-icon">💬</span>
          <span class="chat-text">${settings.widgetTitle}</span>
        </div>
        <div class="chat-window" id="chat-window" style="display: none;">
          <div class="chat-header">
            <span class="chat-title">${settings.widgetTitle}</span>
            <div class="chat-header-actions">
              <button class="chat-new-conversation" id="chat-new-conversation" title="Start new conversation">🔄</button>
              <button class="chat-close" id="chat-close">×</button>
            </div>
          </div>
          <div class="chat-messages" id="chat-messages">
            <div class="welcome-message">
              <p>Hello! How can I help you today?</p>
            </div>
          </div>
          <div class="chat-input-container">
            <input type="text" id="chat-input" placeholder="Type your message..." />
            <button id="chat-send">Send</button>
          </div>
        </div>
      </div>
    `;

    $('body').append(widgetHtml);

    // Bind events
    $('#chat-toggle').on('click', function () {
      $('#chat-window').toggle();
      if ($('#chat-window').is(':visible')) {
        $('#chat-input').focus();
      }
    });

    $('#chat-close').on('click', function () {
      $('#chat-window').hide();
    });

    $('#chat-new-conversation').on('click', function () {
      startNewConversation();
    });

    $('#chat-send').on('click', function () {
      sendMessage();
    });

    $('#chat-input').on('keypress', function (e) {
      if (e.which === 13) { // Enter key
        sendMessage();
      }
    });

    // Load conversation history if enabled and session behavior supports it
    var sessionBehavior = drupalSettings.n8nChat.sessionBehavior || 'refresh';
    var historySupport = drupalSettings.n8nChat.historySupport || false;

    if (historySupport && (sessionBehavior === 'load_history' || sessionBehavior === 'persist')) {
      loadConversationHistory(settings);
    }
  }

  /**
   * Loads conversation history from n8n.
   */
  function loadConversationHistory(settings) {
    var requestData = {
      action: 'getHistory',
      sessionId: getSessionId(),
      userId: drupalSettings.user ? drupalSettings.user.uid : 'anonymous'
    };

    // Debug logging if enabled
    if (settings.debugMode || drupalSettings.n8nChat.debugMode) {
      console.log('n8n Chat - Loading conversation history:', requestData);
    }

    $.ajax({
      url: settings.webhookUrl,
      method: 'POST',
      contentType: 'application/json',
      data: JSON.stringify(requestData),
      success: function (response) {
        // Debug logging if enabled
        if (settings.debugMode || drupalSettings.n8nChat.debugMode) {
          console.log('n8n Chat - History response:', response);
        }

        // Handle history response
        if (response && response.history && Array.isArray(response.history)) {
          // Clear welcome message first
          $('#chat-messages').empty();

          // Add historical messages
          response.history.forEach(function (msg) {
            if (msg.type && msg.message) {
              addMessageToChat(msg.type, msg.message, msg.timestamp);
            }
          });

          // If no history, show welcome message
          if (response.history.length === 0) {
            $('#chat-messages').append(`
              <div class="welcome-message">
                <p>Hello! How can I help you today?</p>
              </div>
            `);
          }
        }
      },
      error: function (xhr, status, error) {
        // Debug logging if enabled
        if (settings.debugMode || drupalSettings.n8nChat.debugMode) {
          console.log('n8n Chat - History request failed (this is normal if n8n doesn\'t support history):', {
            status: status,
            error: error
          });
        }
        // Don't show error to user for history loading - just keep welcome message
      }
    });
  }

  /**
   * Sends a message to the n8n webhook.
   */
  function sendMessage() {
    var message = $('#chat-input').val().trim();
    if (!message) { return;
    }

    var settings = drupalSettings.n8nChat;

    // Add user message to chat
    addMessageToChat('user', message);
    $('#chat-input').val('');

    // Show typing indicator
    addTypingIndicator();

    var requestData = {
      chatInput: message,
      timestamp: new Date().toISOString(),
      sessionId: getSessionId(),
      userId: drupalSettings.user ? drupalSettings.user.uid : 'anonymous'
    };

    // Debug logging if enabled
    if (settings.debugMode || drupalSettings.n8nChat.debugMode) {
      console.log('n8n Chat - Sending request:', requestData);
    }

    // Send to n8n webhook
    $.ajax({
      url: settings.webhookUrl,
      method: 'POST',
      contentType: 'application/json',
      data: JSON.stringify(requestData),
      success: function (response) {
        removeTypingIndicator();

        // Debug logging if enabled
        if (settings.debugMode || drupalSettings.n8nChat.debugMode) {
          console.log('n8n Chat - Response received:', response);
        }

        // Handle response from n8n
        if (response && response.output) {
          addMessageToChat('bot', response.output);
        } else {
          addMessageToChat('bot', 'Thank you for your message. We\'ll get back to you soon!');
          if (settings.debugMode || drupalSettings.n8nChat.debugMode) {
            console.warn('n8n Chat - No output field found in response:', response);
          }
        }
      },
      error: function (xhr, status, error) {
        removeTypingIndicator();

        // Debug logging if enabled
        if (settings.debugMode || drupalSettings.n8nChat.debugMode) {
          console.error('n8n Chat - Request failed:', {
            status: status,
            error: error,
            response: xhr.responseText
          });
        }

        addMessageToChat('bot', 'Sorry, there was an error processing your message. Please try again.');
      }
    });
  }

  /**
   * Adds a message to the chat window.
   */
  function addMessageToChat(sender, message, timestamp) {
    var messageTime;
    if (timestamp) {
      messageTime = new Date(timestamp).toLocaleTimeString();
    } else {
      messageTime = new Date().toLocaleTimeString();
    }

    var messageHtml = `
      <div class="chat-message ${sender}">
        <div class="message-content">${message}</div>
        <div class="message-time">${messageTime}</div>
      </div>
    `;

    $('#chat-messages').append(messageHtml);
    $('#chat-messages').scrollTop($('#chat-messages')[0].scrollHeight);
  }

  /**
   * Adds typing indicator.
   */
  function addTypingIndicator() {
    var typingHtml = `
      <div class="chat-message bot typing-indicator" id="typing-indicator">
        <div class="message-content">
          <span class="typing-dots">
            <span></span>
            <span></span>
            <span></span>
          </span>
        </div>
      </div>
    `;

    $('#chat-messages').append(typingHtml);
    $('#chat-messages').scrollTop($('#chat-messages')[0].scrollHeight);
  }

  /**
   * Removes typing indicator.
   */
  function removeTypingIndicator() {
    $('#typing-indicator').remove();
  }

  /**
   * Starts a new conversation by resetting session ID and clearing messages.
   */
  function startNewConversation() {
    // Reset session ID
    resetSessionId();

    // Clear chat messages
    $('#chat-messages').empty();

    // Add welcome message
    $('#chat-messages').append(`
      <div class="welcome-message">
        <p>Hello! How can I help you today?</p>
      </div>
    `);

    // Show confirmation
    addMessageToChat('system', 'New conversation started. Your previous chat history has been cleared.');

    // Focus on input
    $('#chat-input').focus();
  }

})(jQuery, Drupal, drupalSettings);
