<?php

namespace Drupal\n8n_chat\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure n8n Chat settings for this site.
 */
class N8nChatSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'n8n_chat_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['n8n_chat.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('n8n_chat.settings');

    $form['global_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Global Widget Settings'),
      '#description' => $this->t('Configure global settings for the n8n chat widget. These settings apply when the widget is enabled globally across the site.'),
    ];

    $form['global_settings']['enable_global_widget'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable global chat widget'),
      '#description' => $this->t('When enabled, the chat widget will appear on all pages. You can also place the widget using blocks for more granular control.'),
      '#default_value' => $config->get('enable_global_widget'),
    ];

    $form['global_settings']['webhook_url'] = [
      '#type' => 'url',
      '#title' => $this->t('n8n Webhook URL'),
      '#description' => $this->t('Enter the n8n webhook URL that will handle chat messages. This should be the URL of your n8n workflow webhook.'),
      '#default_value' => $config->get('webhook_url'),
      '#states' => [
        'required' => [
          ':input[name="enable_global_widget"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['appearance'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Appearance Settings'),
    ];

    $form['appearance']['widget_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Widget Title'),
      '#description' => $this->t('The title displayed on the chat widget button and header.'),
      '#default_value' => $config->get('widget_title') ?: $this->t('Chat with us'),
    ];

    $form['appearance']['widget_position'] = [
      '#type' => 'select',
      '#title' => $this->t('Widget Position'),
      '#description' => $this->t('Choose where the chat widget should appear on the page.'),
      '#options' => [
        'bottom-right' => $this->t('Bottom Right'),
        'bottom-left' => $this->t('Bottom Left'),
        'top-right' => $this->t('Top Right'),
        'top-left' => $this->t('Top Left'),
      ],
      '#default_value' => $config->get('widget_position') ?: 'bottom-right',
    ];

    $form['appearance']['widget_theme'] = [
      '#type' => 'select',
      '#title' => $this->t('Widget Theme'),
      '#description' => $this->t('Choose the visual theme for the chat widget.'),
      '#options' => [
        'light' => $this->t('Light'),
        'dark' => $this->t('Dark'),
      ],
      '#default_value' => $config->get('widget_theme') ?: 'light',
    ];

    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced Settings'),
      '#open' => FALSE,
    ];

    $form['advanced']['custom_css'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Custom CSS'),
      '#description' => $this->t('Add custom CSS to style the chat widget. This will be applied globally.'),
      '#default_value' => $config->get('custom_css'),
      '#rows' => 10,
    ];

    $form['advanced']['debug_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable debug mode'),
      '#description' => $this->t('When enabled, additional console logging will be available for troubleshooting.'),
      '#default_value' => $config->get('debug_mode'),
    ];

    $form['integration'] = [
      '#type' => 'details',
      '#title' => $this->t('Integration Settings'),
      '#open' => FALSE,
    ];

    $form['integration']['ai_integration'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable AI Chatbot integration'),
      '#description' => $this->t("When enabled, the widget will integrate with Drupal's AI Chatbot module for enhanced functionality."),
      '#default_value' => $config->get('ai_integration'),
    ];

    $form['integration']['user_context'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Send user context'),
      '#description' => $this->t('When enabled, user information (if logged in) will be sent with chat messages to n8n.'),
      '#default_value' => $config->get('user_context'),
    ];

    $form['integration']['session_behavior'] = [
      '#type' => 'select',
      '#title' => $this->t('Session behavior'),
      '#description' => $this->t('Choose how sessions are handled when the page loads.'),
      '#options' => [
        'refresh' => $this->t('Always start new session (recommended)'),
        'persist' => $this->t('Continue previous session'),
        'load_history' => $this->t('Continue session and load history from n8n'),
      ],
      '#default_value' => $config->get('session_behavior') ?: 'refresh',
    ];

    $form['integration']['history_support'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable conversation history'),
      '#description' => $this->t('When enabled, the widget will attempt to load previous messages from n8n on page load. Your n8n workflow must support the "getHistory" action.'),
      '#default_value' => $config->get('history_support'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $webhook_url = $form_state->getValue('webhook_url');

    // Validate webhook URL if provided.
    if (!empty($webhook_url) && !filter_var($webhook_url, FILTER_VALIDATE_URL)) {
      $form_state->setErrorByName('webhook_url', $this->t('Please enter a valid webhook URL.'));
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('n8n_chat.settings')
      ->set('enable_global_widget', $form_state->getValue('enable_global_widget'))
      ->set('webhook_url', $form_state->getValue('webhook_url'))
      ->set('widget_title', $form_state->getValue('widget_title'))
      ->set('widget_position', $form_state->getValue('widget_position'))
      ->set('widget_theme', $form_state->getValue('widget_theme'))
      ->set('custom_css', $form_state->getValue('custom_css'))
      ->set('debug_mode', $form_state->getValue('debug_mode'))
      ->set('ai_integration', $form_state->getValue('ai_integration'))
      ->set('user_context', $form_state->getValue('user_context'))
      ->set('session_behavior', $form_state->getValue('session_behavior'))
      ->set('history_support', $form_state->getValue('history_support'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
