<?php

namespace Drupal\n8n_chat\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Provides an 'n8n Chat' Block.
 *
 * @Block(
 *   id = "n8n_chat_block",
 *   admin_label = @Translation("n8n Chat"),
 *   category = @Translation("AI and Automation"),
 * )
 */
class N8nChatBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new N8nChatBlock instance.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'webhook_url' => '',
      'widget_title' => $this->t('Chat with us'),
      'widget_position' => 'bottom-right',
      'widget_theme' => 'light',
      'custom_css' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();

    $form['webhook_url'] = [
      '#type' => 'url',
      '#title' => $this->t('n8n Webhook URL'),
      '#description' => $this->t('Enter the n8n webhook URL that will handle chat messages.'),
      '#default_value' => $config['webhook_url'],
      '#required' => TRUE,
    ];

    $form['widget_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Widget Title'),
      '#description' => $this->t('The title displayed on the chat widget.'),
      '#default_value' => $config['widget_title'],
      '#required' => TRUE,
    ];

    $form['widget_position'] = [
      '#type' => 'select',
      '#title' => $this->t('Widget Position'),
      '#description' => $this->t('Choose where the chat widget should appear on the page.'),
      '#options' => [
        'bottom-right' => $this->t('Bottom Right'),
        'bottom-left' => $this->t('Bottom Left'),
        'top-right' => $this->t('Top Right'),
        'top-left' => $this->t('Top Left'),
      ],
      '#default_value' => $config['widget_position'],
    ];

    $form['widget_theme'] = [
      '#type' => 'select',
      '#title' => $this->t('Widget Theme'),
      '#description' => $this->t('Choose the visual theme for the chat widget.'),
      '#options' => [
        'light' => $this->t('Light'),
        'dark' => $this->t('Dark'),
      ],
      '#default_value' => $config['widget_theme'],
    ];

    $form['custom_css'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Custom CSS'),
      '#description' => $this->t('Add custom CSS to style the chat widget.'),
      '#default_value' => $config['custom_css'],
      '#rows' => 5,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockValidate($form, FormStateInterface $form_state) {
    $webhook_url = $form_state->getValue('webhook_url');

    // Validate webhook URL if provided.
    if (!empty($webhook_url) && !filter_var($webhook_url, FILTER_VALIDATE_URL)) {
      $form_state->setErrorByName('settings][webhook_url', $this->t('Please enter a valid webhook URL.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $values = $form_state->getValues();
    $this->configuration['webhook_url'] = $values['webhook_url'];
    $this->configuration['widget_title'] = $values['widget_title'];
    $this->configuration['widget_position'] = $values['widget_position'];
    $this->configuration['widget_theme'] = $values['widget_theme'];
    $this->configuration['custom_css'] = $values['custom_css'];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();

    // Return empty build if webhook URL is not configured.
    if (empty($config['webhook_url'])) {
      return [];
    }

    $build = [
      '#theme' => 'n8n_chat_block',
      '#webhook_url' => $config['webhook_url'],
      '#widget_title' => $config['widget_title'],
      '#widget_position' => $config['widget_position'],
      '#widget_theme' => $config['widget_theme'],
      '#custom_css' => $config['custom_css'],
      '#attached' => [
        'library' => ['n8n_chat/n8n_chat'],
        'drupalSettings' => [
          'n8nChat' => [
            'webhookUrl' => $config['webhook_url'],
            'widgetTitle' => $config['widget_title'],
            'widgetPosition' => $config['widget_position'],
            'widgetTheme' => $config['widget_theme'],
            'debugMode' => $this->configFactory->get('n8n_chat.settings')->get('debug_mode') ?: FALSE,
            'sessionBehavior' => $this->configFactory->get('n8n_chat.settings')->get('session_behavior') ?: 'refresh',
            'historySupport' => $this->configFactory->get('n8n_chat.settings')->get('history_support') ?: FALSE,
          ],
        ],
      ],
    ];

    // Add custom CSS if provided.
    if (!empty($config['custom_css'])) {
      $build['#attached']['html_head'][] = [
        [
          '#tag' => 'style',
          '#value' => $config['custom_css'],
        ],
        'n8n_chat_custom_css',
      ];
    }

    return $build;
  }

}
