<?php

namespace Drupal\Tests\n8n_chat\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the n8n_chat JavaScript functionality.
 *
 * @group n8n_chat
 */
class N8nChatJavascriptTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'node',
    'user',
    'system',
    'n8n_chat',
  ];

  /**
   * A user with administrative permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create an admin user.
    $this->adminUser = $this->drupalCreateUser([
      'administer site configuration',
      'administer blocks',
      'access administration pages',
    ]);

    // Enable global widget for testing.
    $config = $this->config('n8n_chat.settings');
    $config->set('enable_global_widget', TRUE);
    $config->set('webhook_url', 'https://example.com/webhook');
    $config->set('widget_title', 'Test Chat');
    $config->set('widget_position', 'bottom-right');
    $config->set('widget_theme', 'light');
    $config->set('debug_mode', TRUE);
    $config->save();
  }

  /**
   * Tests that the chat widget is rendered on the page.
   */
  public function testChatWidgetRendering() {
    $this->drupalGet('<front>');

    // Wait for the widget to be initialized.
    $this->assertSession()->waitForElement('css', '#n8n-chat');

    // Check that the widget elements are present.
    $this->assertSession()->elementExists('css', '#n8n-chat');
    $this->assertSession()->elementExists('css', '#chat-toggle');
    $this->assertSession()->elementExists('css', '#chat-window');
    $this->assertSession()->elementExists('css', '#chat-messages');
    $this->assertSession()->elementExists('css', '#chat-input');
    $this->assertSession()->elementExists('css', '#chat-send');

    // Check that the widget has the correct classes.
    $widget = $this->assertSession()->elementExists('css', '#n8n-chat');
    $this->assertTrue($widget->hasClass('bottom-right'));
    $this->assertTrue($widget->hasClass('light'));

    // Check that the widget title is displayed.
    $this->assertSession()->pageTextContains('Test Chat');

    // Validate that HTML is properly rendered (not as plain text).
    $chat_toggle = $this->assertSession()->elementExists('css', '#chat-toggle');
    $this->assertTrue($chat_toggle->isVisible(), 'Chat toggle should be visible as a proper HTML element');

    // Check that the welcome message is properly rendered as HTML.
    $welcome_message = $this->assertSession()->elementExists('css', '.welcome-message');
    $this->assertNotEmpty($welcome_message->getText(), 'Welcome message should contain text');
  }

  /**
   * Tests chat widget toggle functionality.
   */
  public function testChatWidgetToggle() {
    $this->drupalGet('<front>');

    // Wait for the widget to be initialized.
    $this->assertSession()->waitForElement('css', '#n8n-chat');

    // Initially, the chat window should be hidden.
    $chat_window = $this->assertSession()->elementExists('css', '#chat-window');
    $this->assertFalse($chat_window->isVisible());

    // Click the toggle button to open the chat.
    $toggle_button = $this->assertSession()->elementExists('css', '#chat-toggle');
    $toggle_button->click();

    // Wait for the chat window to become visible.
    $this->getSession()->wait(1000, 'jQuery("#chat-window").is(":visible")');
    $this->assertTrue($chat_window->isVisible());

    // Click the close button to hide the chat.
    $close_button = $this->assertSession()->elementExists('css', '#chat-close');
    $close_button->click();

    // Wait for the chat window to become hidden.
    $this->getSession()->wait(1000, 'jQuery("#chat-window").is(":hidden")');
    $this->assertFalse($chat_window->isVisible());
  }

  /**
   * Tests chat input focus when opening the widget.
   */
  public function testChatInputFocus() {
    $this->drupalGet('<front>');

    // Wait for the widget to be initialized.
    $this->assertSession()->waitForElement('css', '#n8n-chat');

    // Click the toggle button to open the chat.
    $toggle_button = $this->assertSession()->elementExists('css', '#chat-toggle');
    $toggle_button->click();

    // Wait for the chat window to become visible.
    $this->getSession()->wait(1000, 'jQuery("#chat-window").is(":visible")');

    // Check that the input field receives focus.
    $chat_input = $this->assertSession()->elementExists('css', '#chat-input');
    $this->assertEquals($chat_input, $this->getSession()->getDriver()->getWebDriverSession()->getActiveElement());
  }

  /**
   * Tests new conversation functionality.
   */
  public function testNewConversation() {
    $this->drupalGet('<front>');

    // Wait for the widget to be initialized.
    $this->assertSession()->waitForElement('css', '#n8n-chat');

    // Open the chat window.
    $toggle_button = $this->assertSession()->elementExists('css', '#chat-toggle');
    $toggle_button->click();
    $this->getSession()->wait(1000, 'jQuery("#chat-window").is(":visible")');

    // Add some test content to the chat.
    $this->getSession()->executeScript('
      jQuery("#chat-messages").append("<div class=\"message user-message\">Test message</div>");
    ');

    // Verify the message was added.
    $this->assertSession()->pageTextContains('Test message');

    // Click the new conversation button.
    $new_conversation_button = $this->assertSession()->elementExists('css', '#chat-new-conversation');
    $new_conversation_button->click();

    // Wait a moment for the conversation to be cleared.
    $this->getSession()->wait(500);

    // Check that the chat messages were cleared (only welcome message should remain).
    $this->assertSession()->pageTextNotContains('Test message');
    $this->assertSession()->pageTextContains('Hello! How can I help you today?');
  }

  /**
   * Tests message sending with Enter key.
   */
  public function testMessageSendingWithEnterKey() {
    $this->drupalGet('<front>');

    // Wait for the widget to be initialized.
    $this->assertSession()->waitForElement('css', '#n8n-chat');

    // Open the chat window.
    $toggle_button = $this->assertSession()->elementExists('css', '#chat-toggle');
    $toggle_button->click();
    $this->getSession()->wait(1000, 'jQuery("#chat-window").is(":visible")');

    // Type a message in the input field.
    $chat_input = $this->assertSession()->elementExists('css', '#chat-input');
    $chat_input->setValue('Test message');

    // Press Enter to send the message.
    // Enter key.
    $chat_input->keyPress(13);

    // Wait for the message to be added to the chat.
    $this->getSession()->wait(1000);

    // Check that the message was added to the chat.
    $this->assertSession()->pageTextContains('Test message');

    // Check that the input field was cleared.
    $this->assertEquals('', $chat_input->getValue());
  }

  /**
   * Tests message sending with send button.
   */
  public function testMessageSendingWithButton() {
    $this->drupalGet('<front>');

    // Wait for the widget to be initialized.
    $this->assertSession()->waitForElement('css', '#n8n-chat');

    // Open the chat window.
    $toggle_button = $this->assertSession()->elementExists('css', '#chat-toggle');
    $toggle_button->click();
    $this->getSession()->wait(1000, 'jQuery("#chat-window").is(":visible")');

    // Type a message in the input field.
    $chat_input = $this->assertSession()->elementExists('css', '#chat-input');
    $chat_input->setValue('Button test message');

    // Click the send button.
    $send_button = $this->assertSession()->elementExists('css', '#chat-send');
    $send_button->click();

    // Wait for the message to be added to the chat.
    $this->getSession()->wait(1000);

    // Check that the message was added to the chat.
    $this->assertSession()->pageTextContains('Button test message');

    // Check that the input field was cleared.
    $this->assertEquals('', $chat_input->getValue());
  }

  /**
   * Tests that empty messages are not sent.
   */
  public function testEmptyMessageNotSent() {
    $this->drupalGet('<front>');

    // Wait for the widget to be initialized.
    $this->assertSession()->waitForElement('css', '#n8n-chat');

    // Open the chat window.
    $toggle_button = $this->assertSession()->elementExists('css', '#chat-toggle');
    $toggle_button->click();
    $this->getSession()->wait(1000, 'jQuery("#chat-window").is(":visible")');

    // Try to send an empty message.
    $send_button = $this->assertSession()->elementExists('css', '#chat-send');
    $send_button->click();

    // Wait a moment.
    $this->getSession()->wait(500);

    // Check that no new message was added (only welcome message should be present).
    $messages = $this->getSession()->getPage()->findAll('css', '.message');
    // Only the welcome message.
    $this->assertCount(1, $messages);
  }

  /**
   * Tests drupalSettings configuration.
   */
  public function testDrupalSettingsConfiguration() {
    $this->drupalGet('<front>');

    // Check that drupalSettings contains the expected configuration.
    $settings = $this->getSession()->evaluateScript('return drupalSettings.n8nChat;');

    $this->assertEquals('https://example.com/webhook', $settings['webhookUrl']);
    $this->assertEquals('Test Chat', $settings['widgetTitle']);
    $this->assertEquals('bottom-right', $settings['widgetPosition']);
    $this->assertEquals('light', $settings['widgetTheme']);
    $this->assertTrue($settings['debugMode']);
    $this->assertEquals('refresh', $settings['sessionBehavior']);
    $this->assertFalse($settings['historySupport']);
  }

}
