<?php

namespace Drupal\Tests\n8n_chat\Unit\Form;

use Drupal\Tests\UnitTestCase;

/**
 * Tests the business logic of N8nChatSettingsForm without mocking Drupal types.
 *
 * This follows the "don't mock what you don't own" principle by testing
 * only pure business logic that doesn't depend on Drupal interfaces.
 *
 * @group n8n_chat
 */
class N8nChatSettingsFormUnitTest extends UnitTestCase {

  /**
   * Tests URL validation logic.
   *
   * This tests the same logic used in the form validation
   * without mocking FormStateInterface.
   */
  public function testUrlValidation() {
    // Test valid URLs.
    $valid_urls = [
      'https://example.com/webhook',
      'http://localhost:5678/webhook',
      'https://n8n.example.com/webhook/abc123',
      'https://hooks.zapier.com/hooks/catch/123/abc/',
    ];

    foreach ($valid_urls as $url) {
      $this->assertTrue(
        filter_var($url, FILTER_VALIDATE_URL) !== FALSE,
        "URL '$url' should be valid"
      );
    }

    // Test invalid URLs.
    $invalid_urls = [
      'invalid-url',
      'not-a-url',
      '',  // Empty string should be handled separately
      'javascript:alert(1)',
      'http://',
      'https://',
      'not a url at all',
      'www.example.com',  // Missing protocol
    ];

    foreach ($invalid_urls as $url) {
      $this->assertFalse(
        filter_var($url, FILTER_VALIDATE_URL) !== FALSE,
        "URL '$url' should be invalid"
      );
    }
  }

  /**
   * Tests that empty URLs are handled correctly.
   *
   * Empty URLs should be allowed (webhook is optional).
   */
  public function testEmptyUrlHandling() {
    $empty_values = ['', null, false];

    foreach ($empty_values as $value) {
      $is_empty = empty($value);
      $this->assertTrue($is_empty, 'Empty values should be considered empty');
    }
  }

  /**
   * Tests widget position options.
   */
  public function testWidgetPositionOptions() {
    $expected_positions = [
      'bottom-right',
      'bottom-left',
      'top-right',
      'top-left',
    ];

    // This would be the same array used in the form.
    $form_positions = [
      'bottom-right' => 'Bottom Right',
      'bottom-left' => 'Bottom Left',
      'top-right' => 'Top Right',
      'top-left' => 'Top Left',
    ];

    $this->assertEquals($expected_positions, array_keys($form_positions));
  }

  /**
   * Tests widget theme options.
   */
  public function testWidgetThemeOptions() {
    $expected_themes = ['light', 'dark'];

    $form_themes = [
      'light' => 'Light',
      'dark' => 'Dark',
    ];

    $this->assertEquals($expected_themes, array_keys($form_themes));
  }

  /**
   * Tests session behavior options.
   */
  public function testSessionBehaviorOptions() {
    $expected_behaviors = ['refresh', 'persist', 'load_history'];

    $form_behaviors = [
      'refresh' => 'Always start new session (recommended)',
      'persist' => 'Continue previous session',
      'load_history' => 'Continue session and load history from n8n',
    ];

    $this->assertEquals($expected_behaviors, array_keys($form_behaviors));
  }

  /**
   * Tests default values logic.
   */
  public function testDefaultValues() {
    // Test default widget title logic.
    $config_value = null;
    $default_title = 'Chat with us';
    $widget_title = $config_value ?: $default_title;
    $this->assertEquals($default_title, $widget_title);

    // Test with existing config value.
    $config_value = 'Custom Chat Title';
    $widget_title = $config_value ?: $default_title;
    $this->assertEquals('Custom Chat Title', $widget_title);

    // Test default position.
    $config_position = null;
    $default_position = 'bottom-right';
    $widget_position = $config_position ?: $default_position;
    $this->assertEquals($default_position, $widget_position);

    // Test default theme.
    $config_theme = null;
    $default_theme = 'light';
    $widget_theme = $config_theme ?: $default_theme;
    $this->assertEquals($default_theme, $widget_theme);

    // Test default session behavior.
    $config_behavior = null;
    $default_behavior = 'refresh';
    $session_behavior = $config_behavior ?: $default_behavior;
    $this->assertEquals($default_behavior, $session_behavior);
  }

  /**
   * Tests configuration key validation.
   */
  public function testConfigurationKeys() {
    $expected_config_keys = [
      'enable_global_widget',
      'webhook_url',
      'widget_title',
      'widget_position',
      'widget_theme',
      'custom_css',
      'debug_mode',
      'ai_integration',
      'user_context',
      'session_behavior',
      'history_support',
    ];

    // This would match the keys used in submitForm().
    $form_config_keys = [
      'enable_global_widget',
      'webhook_url',
      'widget_title',
      'widget_position',
      'widget_theme',
      'custom_css',
      'debug_mode',
      'ai_integration',
      'user_context',
      'session_behavior',
      'history_support',
    ];

    $this->assertEquals($expected_config_keys, $form_config_keys);
  }

  /**
   * Tests form ID format.
   */
  public function testFormIdFormat() {
    $expected_form_id = 'n8n_chat_settings';

    // Test that form ID follows Drupal conventions.
    $this->assertMatchesRegularExpression('/^[a-z0-9_]+$/', $expected_form_id);
    $this->assertStringContainsString('n8n_chat', $expected_form_id);
    $this->assertStringContainsString('settings', $expected_form_id);
  }

  /**
   * Tests editable config names format.
   */
  public function testEditableConfigNamesFormat() {
    $expected_config_name = 'n8n_chat.settings';

    // Test that config name follows Drupal conventions.
    $this->assertMatchesRegularExpression('/^[a-z0-9_]+\.[a-z0-9_]+$/', $expected_config_name);
    $this->assertStringStartsWith('n8n_chat.', $expected_config_name);
  }

}
