<?php

namespace Drupal\Tests\n8n_chat\Unit\Helpers;

use Drupal\Tests\UnitTestCase;

/**
 * Tests form validation helper functions.
 *
 * This demonstrates how to extract and test business logic
 * without depending on Drupal form interfaces.
 *
 * @group n8n_chat
 */
class FormValidationHelperTest extends UnitTestCase {

  /**
   * Tests webhook URL validation helper.
   */
  public function testWebhookUrlValidation() {
    // This could be extracted from the form class as a static method.
    $validator = new WebhookUrlValidator();

    // Test valid URLs.
    $this->assertTrue($validator->isValid('https://example.com/webhook'));
    $this->assertTrue($validator->isValid('http://localhost:5678/webhook'));

    // Test invalid URLs.
    $this->assertFalse($validator->isValid('invalid-url'));
    $this->assertFalse($validator->isValid('not-a-url'));

    // Test empty URLs (should be valid - webhook is optional).
    $this->assertTrue($validator->isValid(''));
    $this->assertTrue($validator->isValid(null));
  }

  /**
   * Tests form data sanitization.
   */
  public function testFormDataSanitization() {
    $sanitizer = new FormDataSanitizer();

    // Test CSS sanitization.
    $css_input = '.chat { color: red; } <script>alert("xss")</script>';
    $sanitized_css = $sanitizer->sanitizeCss($css_input);
    $this->assertStringNotContainsString('<script>', $sanitized_css);
    $this->assertStringContainsString('.chat { color: red; }', $sanitized_css);

    // Test title sanitization.
    $title_input = 'Chat <b>Bold</b> Widget';
    $sanitized_title = $sanitizer->sanitizeTitle($title_input);
    $this->assertStringNotContainsString('<b>', $sanitized_title);
    $this->assertEquals('Chat Bold Widget', $sanitized_title);
  }

}

/**
 * Helper class for webhook URL validation.
 *
 * This extracts the validation logic from the form class
 * so it can be tested independently.
 */
class WebhookUrlValidator {

  /**
   * Validates a webhook URL.
   *
   * @param string|null $url
   *   The URL to validate.
   *
   * @return bool
   *   TRUE if valid, FALSE otherwise.
   */
  public function isValid($url) {
    // Empty URLs are valid (webhook is optional).
    if (empty($url)) {
      return TRUE;
    }

    // Use the same validation logic as the form.
    return filter_var($url, FILTER_VALIDATE_URL) !== FALSE;
  }

  /**
   * Gets validation error message.
   *
   * @param string $url
   *   The invalid URL.
   *
   * @return string
   *   The error message.
   */
  public function getErrorMessage($url) {
    return 'Please enter a valid webhook URL.';
  }

}

/**
 * Helper class for form data sanitization.
 */
class FormDataSanitizer {

  /**
   * Sanitizes CSS input.
   *
   * @param string $css
   *   The CSS to sanitize.
   *
   * @return string
   *   The sanitized CSS.
   */
  public function sanitizeCss($css) {
    // Remove script tags and other dangerous content.
    $css = preg_replace('/<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/mi', '', $css);
    return $css;
  }

  /**
   * Sanitizes title input.
   *
   * @param string $title
   *   The title to sanitize.
   *
   * @return string
   *   The sanitized title.
   */
  public function sanitizeTitle($title) {
    // Strip HTML tags.
    return strip_tags($title);
  }

}
