<?php

namespace Drupal\Tests\n8n_chat\Unit\Plugin\Block;

use Prophecy\Argument;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\n8n_chat\Plugin\Block\N8nChatBlock;
use Drupal\Tests\UnitTestCase;
use Prophecy\PhpUnit\ProphecyTrait;

/**
 * Tests the N8nChatBlock.
 *
 * @group n8n_chat
 * @coversDefaultClass \Drupal\n8n_chat\Plugin\Block\N8nChatBlock
 */
class N8nChatBlockTest extends UnitTestCase {

  use ProphecyTrait;

  /**
   * The block under test.
   *
   * @var \Drupal\n8n_chat\Plugin\Block\N8nChatBlock
   */
  protected $block;

  /**
   * The config factory mock.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $configFactory;

  /**
   * The config object mock.
   *
   * @var \Drupal\Core\Config\Config|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->configFactory = $this->prophesize(ConfigFactoryInterface::class);
    $this->config = $this->prophesize(Config::class);

    $this->configFactory->get('n8n_chat.settings')
      ->willReturn($this->config->reveal());

    // Mock string translation.
    $string_translation = $this->prophesize(TranslationInterface::class);
    $string_translation->translate(Argument::any(), Argument::any(), Argument::any())
      ->will(function ($args) {
        return $args[0];
      });

    $configuration = [];
    $plugin_id = 'n8n_chat_block';
    $plugin_definition = [
      'admin_label' => 'n8n Chat',
      'category' => 'AI and Automation',
      'provider' => 'n8n_chat',
    ];

    $this->block = new N8nChatBlock(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $this->configFactory->reveal()
    );

    $this->block->setStringTranslation($string_translation->reveal());
  }

  /**
   * Tests the default configuration.
   *
   * @covers ::defaultConfiguration
   */
  public function testDefaultConfiguration() {
    $expected = [
      'webhook_url' => '',
      'widget_title' => 'Chat with us',
      'widget_position' => 'bottom-right',
      'widget_theme' => 'light',
      'custom_css' => '',
    ];

    $this->assertEquals($expected, $this->block->defaultConfiguration());
  }

  /**
   * Tests the block form building.
   *
   * @covers ::blockForm
   */
  public function testBlockForm() {
    $form = [];
    $form_state = $this->prophesize(FormStateInterface::class);

    $result = $this->block->blockForm($form, $form_state->reveal());

    // Test that all expected form elements are present.
    $this->assertArrayHasKey('webhook_url', $result);
    $this->assertEquals('url', $result['webhook_url']['#type']);
    $this->assertEquals('', $result['webhook_url']['#default_value']);

    $this->assertArrayHasKey('widget_title', $result);
    $this->assertEquals('textfield', $result['widget_title']['#type']);
    $this->assertEquals('Chat with us', $result['widget_title']['#default_value']);

    $this->assertArrayHasKey('widget_position', $result);
    $this->assertEquals('select', $result['widget_position']['#type']);
    $this->assertEquals('bottom-right', $result['widget_position']['#default_value']);

    $this->assertArrayHasKey('widget_theme', $result);
    $this->assertEquals('select', $result['widget_theme']['#type']);
    $this->assertEquals('light', $result['widget_theme']['#default_value']);

    $this->assertArrayHasKey('custom_css', $result);
    $this->assertEquals('textarea', $result['custom_css']['#type']);
    $this->assertEquals('', $result['custom_css']['#default_value']);
  }

  /**
   * Tests block form submission.
   *
   * @covers ::blockSubmit
   */
  public function testBlockSubmit() {
    $form = [];
    $form_state = $this->prophesize(FormStateInterface::class);

    $form_state->getValue('webhook_url')->willReturn('https://example.com/webhook');
    $form_state->getValue('widget_title')->willReturn('Test Chat');
    $form_state->getValue('widget_position')->willReturn('bottom-left');
    $form_state->getValue('widget_theme')->willReturn('dark');
    $form_state->getValue('custom_css')->willReturn('.test { color: red; }');

    $this->block->blockSubmit($form, $form_state->reveal());

    $configuration = $this->block->getConfiguration();
    $this->assertEquals('https://example.com/webhook', $configuration['webhook_url']);
    $this->assertEquals('Test Chat', $configuration['widget_title']);
    $this->assertEquals('bottom-left', $configuration['widget_position']);
    $this->assertEquals('dark', $configuration['widget_theme']);
    $this->assertEquals('.test { color: red; }', $configuration['custom_css']);
  }

  /**
   * Tests block validation with invalid webhook URL.
   *
   * @covers ::blockValidate
   */
  public function testBlockValidateWithInvalidWebhookUrl() {
    $form = [
      'webhook_url' => [
        '#parents' => ['settings', 'webhook_url'],
      ],
    ];
    $form_state = $this->prophesize(FormStateInterface::class);

    $form_state->getValue('webhook_url')->willReturn('invalid-url');
    $form_state->setErrorByName('settings][webhook_url', 'Please enter a valid webhook URL.')->shouldBeCalled();

    $this->block->blockValidate($form, $form_state->reveal());
  }

  /**
   * Tests block validation with valid webhook URL.
   *
   * @covers ::blockValidate
   */
  public function testBlockValidateWithValidWebhookUrl() {
    $form = [
      'webhook_url' => [
        '#parents' => ['settings', 'webhook_url'],
      ],
    ];
    $form_state = $this->prophesize(FormStateInterface::class);

    $form_state->getValue('webhook_url')->willReturn('https://example.com/webhook');
    $form_state->setErrorByName('settings][webhook_url', 'Please enter a valid webhook URL.')->shouldNotBeCalled();

    $this->block->blockValidate($form, $form_state->reveal());
  }

  /**
   * Tests block building.
   *
   * @covers ::build
   */
  public function testBuild() {
    // Set up configuration.
    $this->block->setConfiguration([
      'webhook_url' => 'https://example.com/webhook',
      'widget_title' => 'Test Chat',
      'widget_position' => 'bottom-left',
      'widget_theme' => 'dark',
      'custom_css' => '.test { color: red; }',
    ]);

    // Set up config factory expectations.
    $this->config->get('debug_mode')->willReturn(TRUE);
    $this->config->get('session_behavior')->willReturn('persist');
    $this->config->get('history_support')->willReturn(FALSE);

    $build = $this->block->build();

    // Test the build array structure.
    $this->assertEquals('n8n_chat_block', $build['#theme']);
    $this->assertEquals('https://example.com/webhook', $build['#webhook_url']);
    $this->assertEquals('Test Chat', $build['#widget_title']);
    $this->assertEquals('bottom-left', $build['#widget_position']);
    $this->assertEquals('dark', $build['#widget_theme']);
    $this->assertEquals('.test { color: red; }', $build['#custom_css']);

    // Test attached libraries and settings.
    $this->assertArrayHasKey('#attached', $build);
    $this->assertContains('n8n_chat/n8n_chat', $build['#attached']['library']);
    $this->assertArrayHasKey('drupalSettings', $build['#attached']);
    $this->assertArrayHasKey('n8nChat', $build['#attached']['drupalSettings']);

    $settings = $build['#attached']['drupalSettings']['n8nChat'];
    $this->assertEquals('https://example.com/webhook', $settings['webhookUrl']);
    $this->assertEquals('Test Chat', $settings['widgetTitle']);
    $this->assertEquals('bottom-left', $settings['widgetPosition']);
    $this->assertEquals('dark', $settings['widgetTheme']);
    $this->assertEquals(TRUE, $settings['debugMode']);
    $this->assertEquals('persist', $settings['sessionBehavior']);
    $this->assertEquals(FALSE, $settings['historySupport']);
  }

  /**
   * Tests block building with empty webhook URL.
   *
   * @covers ::build
   */
  public function testBuildWithEmptyWebhookUrl() {
    // Set up configuration with empty webhook URL.
    $this->block->setConfiguration([
      'webhook_url' => '',
      'widget_title' => 'Test Chat',
      'widget_position' => 'bottom-right',
      'widget_theme' => 'light',
      'custom_css' => '',
    ]);

    $build = $this->block->build();

    // Should return empty array when webhook URL is not configured.
    $this->assertEquals([], $build);
  }

}
